import cranium
import multiprocessing as mp
import time
import os
import sys
from pushbullet import Pushbullet
from functools import partial
from random import randint


genthresh = 0.5
medthresh = 0.25
radius = 20
microns = [0.16,0.16,0.21]
deg = 2
comporder = [0,2,1]
fitdim = ['x','z']
scale = [1,1,1]

columns = ['x','y','z','ac','r','theta']

def process(num,expname=None,outdir=None,gtype=None,adir=None,zdir=None):
	tic = time.time()

	e = cranium.embryo(expname,num,outdir)

	e.add_channel(os.path.join(adir,'AT_'+num+'_Probabilities.h5'),'AT')
	e.add_channel(os.path.join(zdir,gtype+'_'+num+'_Probabilities.h5'),gtype)
	e.chnls['AT'].preprocess_data(genthresh,[1,1,1],microns)
	e.chnls[gtype].preprocess_data(genthresh,[1,1,1],microns)

	e.chnls['AT'].calculate_pca_median(e.chnls['AT'].raw_data,medthresh,radius,microns)
	pca = e.chnls['AT'].pcamed
	e.chnls['AT'].pca_transform_3d(e.chnls['AT'].df_thresh,pca,comporder,fitdim,deg=2)

	mm = e.chnls['AT'].mm
	vertex = e.chnls['AT'].vertex

	e.chnls[gtype].pca_transform_3d(e.chnls[gtype].df_thresh,pca,comporder,fitdim,deg=2,mm=mm,vertex=vertex)

	e.chnls['AT'].transform_coordinates()
	e.chnls[gtype].transform_coordinates()

	e.save_psi()

	toc = time.time()
	print(num,toc-tic)

if __name__=='__main__':

	zrf2 = [
		"D:\\zrfs\\ZRF2\\wt\\AT",
		"D:\\zrfs\\ZRF2\\wt\\ZRF2",
		"D:\\zrfs\\ZRF2\\wt"
	]
	
	zrf3 = [
		"D:\\zrfs\\ZRF3\\AT",
		"D:\\zrfs\\ZRF3\\ZRF3",
		"D:\\zrfs\\ZRF3"
	]

	zrf4 = [
		"D:\\zrfs\\ZRF4\\wt\\AT",
		"D:\\zrfs\\ZRF4\\wt\\ZRF4",
		"D:\\zrfs\\ZRF4\\wt"
	]

	Lgtype = ['zrf2','zrf3','zrf4']
	names = ['wtzrf2','wtzrf3','wtzrf4']
	dirset = [zrf2,zrf3,zrf4]

	for i,dirs in enumerate(dirset):
		name = names[i]
		adir,zdir,root = dirs[0],dirs[1],dirs[2]

		#Create outdir
		outdir = os.path.join(root,'11-20_Output'+str(randint(10,99)))
		os.mkdir(outdir)
		print('outdir',outdir)

		processfxn = partial(process,expname=name,outdir=outdir,gtype=Lgtype[i],adir=adir,zdir=zdir)

		nums = []
		for f in os.listdir(adir):
			if 'Probabilities' in f and 'h5' in f:
				print(f)
				nums.append(f.split('_')[1])

		print(nums)

		n = len(nums)
		for i in range(0,n,5):
			if i+5>n:
				L = nums[i:n]
			else:
				L = nums[i:i+5]

			# if i >= 5: break

			pool = mp.Pool()
			pool.map(processfxn,L)
			pool.close()
			pool.join()


		print('complete',root)


from __future__ import print_function
from builtins import zip
from builtins import str
from builtins import range
__author__ = 'Hans Petter Langtangen <hpl@simula.no>'

#example
#python .\ipynb_generator.py datareport.aipynb test.ipynb WTDIR="C:\\Users\\zfishlab\\Desktop\\zrf1wt13umyot21um\\wt\\Output69semifinal" MUTDIR="C:\\Users\\zfishlab\\Desktop\\zrf1wt13umyot21um\\yot\\Output92semifinal" DTYPE='AT' MYVAR=4 GRADE='excellent'
# C:\Users\zfishlab\Desktop\zrf1wt13umyot21um\Correct_alignment

import sys, os, re, logging

# Mapping of shortnames like py to full language
# name like python used by markdown/pandoc
shortname2language = dict(
    py='Python', ipy='Python', pyshell='Python', cy='Python',
    c='C', cpp='Cpp', f='Fortran', f95='Fortran95',
    rb='Ruby', pl='Perl', sh='Shell', js='JavaScript', html='HTML',
    tex='Tex', sys='Bash',
    )

def read(text, argv=sys.argv[2:]):
    lines = text.splitlines()
    # First read all include statements
    for i in range(len(lines)):
        if lines[i].startswith('#include "'):
            filename = lines[i].split('"')[1]
            with open(filename, 'r') as f:
                include_text = f.read()
            lines[i] = include_text
    text = '\n'.join(lines)
    logging.info('******* text after include:\n%s' % text)

    # Run Mako
    mako_kwargs = {}
    for arg in argv:
        key, value = arg.split('=')
        mako_kwargs[key] = value

    encoding = 'utf-8'
    try:
        import mako
        has_mako = True
    except ImportError:
        print('Cannot import mako - mako is not run')
        has_mako = False

    if has_mako:
        from mako.template import Template
        from mako.lookup import TemplateLookup
        lookup = TemplateLookup(directories=[os.curdir])

        text = str(text)
        temp = Template(text=text, lookup=lookup,
                        strict_undefined=True)
        logging.info('******* mako_kwargs: %s' % str(mako_kwargs))
        text = temp.render(**mako_kwargs)

    logging.info('******* text after mako:\n%s' % text)

    # Parse the cells
    lines = text.splitlines()
    cells = []
    inside = None    # indicates which type of cell we are inside
    fullname = None  # full language name in code cells
    for line in lines:
        if line.startswith('-----'):
            # New cell, what type?
            m = re.search(r'-----([a-z0-9-]+)?', line)
            if m:
                shortname = m.group(1)
                if shortname:
                    # Check if code is to be typeset as static
                    # Markdown code (e.g., shortname=py-t)
                    logging.info('******* found shortname %s'
                                 % shortname)
                    astext = shortname[-2:] == '-t'
                    logging.info(
                        '******* cell: astext=%s shortname=%s'
                        % (astext, shortname))
                    if astext:
                        # Markdown
                        shortname = shortname[:-2]
                        inside = 'markdown'
                        cells.append(['markdown', 'code', ['\n']])
                        cells[-1][2].append('```%s\n' % fullname)
                    else:
                        # Code cell
                        if shortname in shortname2language:
                            fullname = shortname2language[shortname]
                        inside = 'codecell'
                        cells.append(['codecell', fullname, []])
                else:
                    logging.info('******* cell: markdown')
                    # Markdown cell
                    inside = 'markdown'
                    cells.append(['markdown', 'text', ['\n']])
            else:
                raise SyntaxError(
                    'Wrong syntax of cell delimiter:\n%s'
                    % repr(line))
        else:
            # Ordinary line in a cell
            if inside in ('markdown', 'codecell'):
                cells[-1][2].append(line)
            else:
                raise SyntaxError(
                    'line\n  %s\nhas not beginning cell delimiter'
                    % line)
    # Merge the lines in each cell to a string
    for i in range(len(cells)):
        if cells[i][0] == 'markdown' and cells[i][1] == 'code':
            # Add an ending ``` of code
            cells[i][2].append('```\n')
        cells[i][2] = '\n'.join(cells[i][2])
    import pprint
    logging.info('******* cell data structure:\b%s'
                 % pprint.pformat(cells))
    return cells

def write(cells, nb_version=4):
    """Turn cells list into valid IPython notebook code."""
    # Use IPython.nbformat functionality for writing the notebook
    
    Dmetadata = {
        "anaconda-cloud": {},
        "kernalspec": {
            "display_name": "Python [Root]",
            "language": "python",
            "name": "Python [Root]"
        },
        "language_info": {
            "codemirror_mode": {
            "name": "ipython",
            "version": 3
            }
        },
        "file_extension": ".py",
        "mimetype": "test/x-python",
        "name": "python",
        "nbconvert_exporter": "python",
        "pygments_lexer": "ipython3",
        "version": "3.5.2"
    }

    if nb_version == 3:
        from nbformat.v3 import (
            new_code_cell, new_text_cell, new_worksheet,
            new_notebook, new_metadata, new_author)
        nb = new_worksheet()

    elif nb_version == 4:
        from nbformat.v4 import (
            new_code_cell, new_markdown_cell, new_notebook)
        nb_cells = []

    for cell_tp, language, block in cells:
        if cell_tp == 'markdown':
            if nb_version == 3:
                nb.cells.append(
                    new_text_cell(u'markdown', source=block))
            elif nb_version == 4:
                nb_cells.append(
                    new_markdown_cell(source=block))
        elif cell_tp == 'codecell':
            if nb_version == 3:
                nb.cells.append(new_code_cell(input=block))
            elif nb_version == 4:
                nb_cells.append(new_code_cell(source=block))

    if nb_version == 3:
        nb = new_notebook(worksheets=[nb], metadata=new_metadata())
        # Let us make v4 notebook here by upgrading
        from nbformat.v4 import upgrade
        nb = upgrade(nb)
        import nbformat.v4.nbjson as nbjson
        # Convert nb to json format
        filestr = nbjson.writes(nb)
    elif nb_version == 4:
        nb = new_notebook(cells=nb_cells, metadata=Dmetadata)
        from nbformat import writes
        filestr = writes(nb, version=4)
    return filestr

def driver():
    """Compile a document and its variables."""
    try:
        filename = sys.argv[1]
        with open(filename, 'r') as f:
            text = f.read()
    except (IndexError, IOError) as e:
        print('Usage: %s filename' % (sys.argv[0]))
        print(e)
        sys.exit(1)
    cells = read(text, argv=sys.argv[3:])
    filestr = write(cells, 3)
    outname = sys.argv[2]
    with open(outname, 'w') as f:
        f.write(filestr)


def test_notebook_generator():
    """Unit test for nosetests or py.test."""
    argv = ['NAME=Core Dump',
            'ADDRESS=Seg. Fault Ltd and Univ. of C. Space',
            'IC=2']
    with open('.test1.aipynb', 'r') as f:
        text = f.read()
    logging.info('******* original text:\n%s' % text)
    cells = read(text, argv=argv)
    computed = write(cells, 3).strip()
    print(computed)

    with open('.test1.ipynb', 'r') as f:
        expected = f.read().strip()

    for c1, c2, n in zip(computed, expected, list(range(len(expected)))):
        if c1 != c2:
            print('character no.%d differ: %s vs %s' % (n, c1, c2))
    assert computed == expected

if __name__ == '__main__':
    logfile = 'tmp.log'
    if os.path.isfile(logfile):
        os.remove(logfile)
    logging.basicConfig(format='%(message)s', level=logging.DEBUG,
                        filename=logfile)
    driver()


import os
import numpy as np
import h5py
import pandas as pd
from sklearn.decomposition import PCA
from skimage.filters import median
from skimage.morphology import disk
import sympy as sp
from scipy.spatial import KDTree
import time

class brain:
	''' Object to manage biological data and associated functions. '''

	def __init__(self):
		'''Initialize brain object'''

	def setup_test_data(self,size=None,gthresh=0.5,scale=[1,1,1],microns=[0.16,0.16,0.21],mthresh=0.2,radius=20,comp_order=[0,2,1],fit_dim=['x','z'],deg=2):
		'''Setup a test dataset to use for testing transform coordinates
		:param int size: Number of points to sample for the test dataset
		'''

		self.read_data(os.path.abspath('..\data\C1\AT_01_Probabilities.h5'))
		self.preprocess_data(gthresh,scale,microns)
		self.calculate_pca_median(self.raw_data,mthresh,radius,microns)
		self.pca_transform_3d(self.df_thresh,self.pcamed,comp_order,fit_dim,deg=deg)
		self.mm = self.fit_model(self.df_align,2,['x','z'])
		if size!= None:
			self.df_align = self.df_align.sample(size)

	def read_data(self,filepath):
		'''
		Reads 3D data from file and selects appropriate channel based on the
		assumption that the channel with the most zeros has zero as the value for no signal

		:param str filepath: Filepath to hdf5 probability file
		:return: Creates the variable :attr:`brain.raw_data`

		.. py:attribute:: brain.raw_data

			Array of shape [z,y,x] containing raw probability data
		'''

		#Read h5 file and extract probability data
		f = h5py.File(filepath,'r')

		#Select either channe0/1 or exported_data
		d = f.get('exported_data')
		if d != None:
			c1 = np.array(d[:,:,:,0])
			c2 = np.array(d[:,:,:,1])
		elif d == None:
			c1 = np.array(f.get('channel0'))
			c2 = np.array(f.get('channel1'))

		#Figure out which channels has more zeros and therefore is background
		if np.count_nonzero(c1<0.1) > np.count_nonzero(c1>0.9):
			#: Array of shape [z,y,x] containing raw probability data
			self.raw_data = c2
		else:
			#: Array of shape [z,y,x] containing raw probability data
			self.raw_data = c1

	def create_dataframe(self,data,scale):
		'''
		Creates a pandas dataframe containing the x,y,z and signal/probability value for each point in the :py:attr:`brain.raw_data` array

		:param array data: Raw probability data in 3D array
		:param array scale: Array of length three containing the micron values for [x,y,z]
		:return: Pandas DataFrame with xyz and probability value for each point
		'''

		#NB: scale variable actually contains microns dimensions

		dim = data.shape
		xyz = np.zeros((dim[0],dim[1],dim[2],4))

		#Generate array with xyz values for each point
		for x in range(dim[2]):

			xyz[:,:,x,2] = x
			xyz[:,:,x,3] = data[:,:,x]

			zindex = np.arange(0,dim[0],1)
			yindex = np.arange(0,dim[1],1)
			gy,gz = np.meshgrid(yindex,zindex)

			xyz[:,:,x,0] = gz
			xyz[:,:,x,1] = gy

		flat = np.reshape(xyz,(-1,4))

		#Create dataframe of points and scale to microns
		df = pd.DataFrame({'x':flat[:,2]*scale[0],'y':flat[:,1]*scale[1],'z':flat[:,0]*scale[2],'value':flat[:,3]})
		return(df)

	def plot_projections(self,df,subset):
		'''
		Plots the x, y, and z projections of the input dataframe in a matplotlib plot

		:param pd.DataFrame df: Dataframe with columns: 'x','y','z'
		:param float subset: Value between 0 and 1 indicating what percentage of the df to subsample
		:returns: Matplotlib figure with three labeled scatterplots
		'''

		df = df.sample(frac=subset)

    	#Create figure and subplots
		fig = plt.figure(figsize=(12,6))
		ax = fig.add_subplot(131)
		ay = fig.add_subplot(132)
		az = fig.add_subplot(133)

		#Create scatter plot for each projection
		ax.scatter(df.x,df.z)
		ay.scatter(df.x,df.y)
		az.scatter(df.z,df.y)

		#Plot model
		xvalues = np.arange(np.min(df.x),np.max(df.x))
		ax.plot(xvalues,self.mm.p(xvalues),c='y')

		#Add labels
		ax.set_title('Y projection')
		ay.set_title('Z projection')
		az.set_title('X projection')
		ax.set_xlabel('X')
		ax.set_ylabel('Z')
		ay.set_xlabel('X')
		ay.set_ylabel('Y')
		az.set_xlabel('Z')
		az.set_ylabel('Y')

		#Adjust spacing and show plot
		plt.subplots_adjust(wspace=0.4)

		return(fig)

	def preprocess_data(self,threshold,scale,microns):
		'''
		Thresholds and scales data prior to PCA

		Creates :py:attr:`brain.threshold`, :py:attr:`brain.df_thresh`, and :py:attr:`brain.df_scl`

		:param float threshold: Value between 0 and 1 to use as a cutoff for minimum pixel value
		:param array scale: Array with three values representing the constant by which to multiply x,y,z respectively
		:param array microns: Array with three values representing the x,y,z micron dimensions of the voxel

		.. py:attribute:: brain.threshold

			Value used to threshold the data prior to calculating the model

		.. py:attribute:: brain.df_thresh

			Dataframe containing only points with values above the specified threshold

		.. py:attribute:: brain.df_scl

			Dataframe containing data from :py:attr:`brain.df_thresh` after a scaling value has been applied
		'''

		#: Dataframe with four columns: x,y,z,value with all points in :py:attr:`brain.raw_data`
		self.df = self.create_dataframe(self.raw_data,microns)

		#Create new dataframe with values above threshold
		self.threshold = threshold
		self.df_thresh = self.df[self.df.value < self.threshold]

		#Scale xyz by value in scale array to force PCA axis selection
		self.scale = scale
		self.df_scl = pd.DataFrame({
			'x':self.df_thresh.x * self.scale[0],
			'y':self.df_thresh.y * self.scale[1],
			'z':self.df_thresh.z * self.scale[2]})

	def process_alignment_data(self,data,threshold,radius,microns):
		'''
		Applies a median filter twice to the data which is used for alignment

		Ensures than any noise in the structural data does not interfere with alignment

		:param array data: Raw data imported by the function :py:func:`brain.read_data`
		:param float threshold: Value between 0 and 1 to use as a cutoff for minimum pixel value
		:param int radius: Integer that determines the radius of the circle used for the median filter
		:param array microns: Array with three values representing the x,y,z micron dimensions of the voxel
		:returns: Dataframe containing data processed with the median filter and threshold
		'''

		#Iterate over each plane and apply median filter twice
		out = np.zeros(data.shape)
		for z in range(data.shape[0]):
			out[z] = median(median(data[z],disk(radius)),disk(radius))

		outdf = self.create_dataframe(out,microns)

		# Changed 7/5/18
		# Skimage filter changes datatype to 0-255 range from 0-1
		# Modification to threshold allows consistent use of threshold based on 0-1 range
		thresh = outdf[outdf.value < 255*(1-threshold)]
		return(thresh)

	def calculate_pca_median(self,data,threshold,radius,microns):
		'''
		Calculate PCA transformation matrix, :py:attr:`brain.pcamed`, based on data (:py:attr:`brain.pcamed`) after applying median filter and threshold

		:param array data: 3D array containing raw probability data
		:param float threshold: Value between 0 and 1 indicating the lower cutoff for positive signal
		:param int radius: Radius of neighborhood that should be considered for the median filter
		:param array microns: Array with three values representing the x,y,z micron dimensions of the voxel

		.. py:attribute:: brain.median

			Pandas dataframe containing data that has been processed with a median filter twice and thresholded

		.. py:attribute:: brain.pcamed

			PCA object managing the transformation matrix and any resulting transformations

		'''

		self.median = self.process_alignment_data(data,threshold,radius,microns)

		self.pcamed = PCA()
		self.pcamed.fit(self.median[['x','y','z']])

	def calculate_pca_median_2d(self,data,threshold,radius,microns):
		'''
		Calculate PCA transformation matrix for 2 dimensions of data, :py:attr:`brain.pcamed`, based on data after applying median filter and threshold

		.. warning:: `fit_dim` is not used to determine which dimensions to fit. Defaults to x and z

		:param array data: 3D array containing raw probability data
		:param float threshold: Value between 0 and 1 indicating the lower cutoff for positive signal
		:param int radius: Radius of neighborhood that should be considered for the median filter
		:param array microns: Array with three values representing the x,y,z micron dimensions of the voxel
		'''

		self.median = self.process_alignment_data(data,threshold,radius,microns)

		self.pcamed = PCA()
		self.pcamed.fit(self.median[['y','z']])

	def pca_transform_2d(self,df,pca,comp_order,fit_dim,deg=2,mm=None,vertex=None,flip=None):
		'''
		Transforms `df` in 2D based on the PCA object, `pca`, whose transformation matrix has already been calculated

		Calling :py:func:`brain.align_data` creates :py:attr:`brain.df_align`

		.. warning:: `fit_dim` is not used to determine which dimensions to fit. Defaults to x and z

		:param pd.DataFrame df: Dataframe containing thresholded xyz data
		:param pca_object pca: A pca object containing a transformation object, e.g. :py:attr:`brain.pcamed`
		:param array comp_order: Array specifies the assignment of components to x,y,z. Form [x component index, y component index, z component index], e.g. [0,2,1]
		:param array fit_dim: Array of length two containing two strings describing the first and second axis for fitting the model, e.g. ['x','z']
		:param int deg: (or None) Degree of the function that should be fit to the model. deg=2 by default
		:param mm: (:py:class:`math_model` or None) Math model for primary channel
		:param array vertex: (or None) Array of type [vx,vy,vz] (:py:attr:`brain.vertex`) indicating the translation values
		:param Bool flip: (or None) Boolean value to determine if the data should be rotated by 180 degrees
		'''

		fit = pca.transform(df[['y','z']])
		df_fit = pd.DataFrame({
			'x':df.x,
			'y':fit[:,comp_order[1]-1],
			'z':fit[:,comp_order[2]-1]
			})

		self.align_data(df_fit,fit_dim,deg=2,mm=None,vertex=None,flip=None)

	def pca_transform_3d(self,df,pca,comp_order,fit_dim,deg=2,mm=None,vertex=None,flip=None):
		'''
		Transforms `df` in 3D based on the PCA object, `pca`, whose transformation matrix has already been calculated

		:param pd.DataFrame df: Dataframe containing thresholded xyz data
		:param pca_object pca: A pca object containing a transformation object, e.g. :py:attr:`brain.pcamed`
		:param array comp_order: Array specifies the assignment of components to x,y,z. Form [x component index, y component index, z component index], e.g. [0,2,1]
		:param array fit_dim: Array of length two containing two strings describing the first and second axis for fitting the model, e.g. ['x','z']
		:param int deg: (or None) Degree of the function that should be fit to the model. deg=2 by default
		:param mm: (:py:class:`math_model` or None) Math model for primary channel
		:param array vertex: (or None) Array of type [vx,vy,vz] (:py:attr:`brain.vertex`) indicating the translation values
		:param Bool flip: (or None) Boolean value to determine if the data should be rotated by 180 degrees
		'''

		fit = pca.transform(df[['x','y','z']])
		df_fit = pd.DataFrame({
			'x':fit[:,comp_order[0]],
			'y':fit[:,comp_order[1]],
			'z':fit[:,comp_order[2]]
			})

		self.align_data(df_fit,fit_dim,deg=2,mm=None,vertex=None,flip=None)

	def align_data(self,df_fit,fit_dim,deg=2,mm=None,vertex=None,flip=None):
		'''
		Apply PCA transformation matrix and align data so that the vertex is at the origin

		Creates :py:attr:`brain.df_align` and :py:attr:`brain.mm`

		:param pd.DataFrame df: dataframe containing thresholded xyz data
		:param array comp_order: Array specifies the assignment of components to x,y,z. Form [x component index, y component index, z component index], e.g. [0,2,1]
		:param array fit_dim: Array of length two containing two strings describing the first and second axis for fitting the model, e.g. ['x','z']
		:param int deg: (or None) Degree of the function that should be fit to the model. deg=2 by default
		:param mm: (:py:class:`math_model` or None) Math model for primary channel
		:param array vertex: (or None) Array of type [vx,vy,vz] (:py:attr:`brain.vertex`) indicating the translation values
		:param Bool flip: (or None) Boolean value to determine if the data should be rotated by 180 degrees

		.. py:attribute:: brain.df_align

			Dataframe containing point data aligned using PCA

		.. py:attribute:: brain.mm

			Math model object fit to data in brain object
		'''

		#If vertex for translation is not included
		if vertex == None and deg==2:
			#Calculate model
			model = np.polyfit(df_fit[fit_dim[0]],df_fit[fit_dim[1]],deg=deg)
			p = np.poly1d(model)

			#Find vertex
			a = -model[1]/(2*model[0])
			if fit_dim[0] == 'x':
				vx = a
				if fit_dim[1] == 'y':
					vy = p(vx)
					vz = df_fit.z.mean()
				else:
					vz = p(vx)
					vy = df_fit.y.mean()
			elif fit_dim[0] == 'y':
				vy = a
				if fit_dim[1] == 'x':
					vx = p(a)
					vz = df_fit.z.mean()
				else:
					vz = p(a)
					vx = df_fit.x.mean()
			elif fit_dim[0] == 'z':
				vz = a
				if fit_dim[1] == 'x':
					vx = p(a)
					vy = df_fit.y.mean()
				else:
					vy = p(a)
					vx = df_fit.x.mean()
			self.vertex = [vx,vy,vz]
		elif deg == 1:
			#Calculate model
			model = np.polyfit(df_fit[fit_dim[0]],df_fit[fit_dim[1]],deg=deg)
			p = np.poly1d(model)

			if fit_dim[0] == 'x':
				vx = df_fit.x.mean()
				if fit_dim[1] == 'y':
					vy = p(vx)
					vz = df_fit.z.mean()
				else:
					vz = p(vx)
					vy = df_fit.y.mean()
			elif fit_dim[0] == 'y':
				vy = df_fit.y.mean()
				if fit_dim[1] == 'x':
					vx = p(vy)
					vz = df_fit.z.mean()
				else:
					vz = p(vy)
					vx = df_fit.x.mean()
			elif fit_dim[0] == 'z':
				vz = df_fit.z.mean()
				if fit_dim[1] == 'x':
					vx = p(vz)
					vy = df_fit.y.mean()
				else:
					vy = p(vz)
					vx = df_fit.x.mean()
			self.vertex = [vx,vy,vz]
		else:
			self.vertex = vertex

		#Translate data so that the vertex is at the origin
		self.df_align = pd.DataFrame({
			'x': df_fit.x - self.vertex[0],
			'y': df_fit.y - self.vertex[1],
			'z': df_fit.z - self.vertex[2]
			})

		#Rotate data by 180 degrees if necessary
		if flip == None or flip == False:
			#Calculate model
			model = np.polyfit(df_fit[fit_dim[0]],df_fit[fit_dim[1]],deg=deg)
			p = np.poly1d(model)

			#If a is less than 0, rotate data
			if model[0] < 0:
				self.df_align = self.flip_data(self.df_align)
		elif flip == True:
			self.df_align = self.flip_data(self.df_align)

		#Calculate final math model
		if mm == None:
			self.mm = self.fit_model(self.df_align,deg,fit_dim)
		else:
			self.mm = mm

	def flip_data(self,df):
		'''
		Rotate data by 180 degrees

		:param dataframe df: Pandas dataframe containing x,y,z data
		:returns: Rotated dataframe
		'''

		r = np.array([[np.cos(np.pi),0,np.sin(np.pi)],
			[0,1,0],
			[-np.sin(np.pi),0,np.cos(np.pi)]])

		rot = np.dot(np.array(df),r)

		dfr = pd.DataFrame({'x':rot[:,0],'y':rot[:,1],'z':rot[:,2]})
		return(dfr)

	def fit_model(self,df,deg,fit_dim):
		'''Fit model to dataframe

		:param pd.DataFrame df: Dataframe containing at least x,y,z
		:param int deg: Degree of the function that should be fit to the model
		:param array fit_dim: Array of length two containing two strings describing the first and second axis for fitting the model, e.g. ['x','z']
		:returns: math model
		:rtype: :py:class:`math_model`
		'''

		mm = math_model(np.polyfit(df[fit_dim[0]], df[fit_dim[1]], deg=deg))
		return(mm)

	def setup_tree(self,xmin,xmax,xstep,rmax,rnum,acf):

		from sympy.abc import alpha,rho
		a,drho = sp.symbols('a drho')
		# s position as a function of alpha and rho
		sp_s = alpha + rho
		F_s = sp.lambdify((alpha,rho),sp_s,'numpy')
		# t position as a function of alpha and rho
		sp_t = a*alpha**2 - rho/(2*a*alpha)
		F_t = sp.lambdify((a,alpha,rho),sp_t,'numpy')
		#Rho step as a function of drho
		sp_rstep = sp.sqrt( drho*4*a**2*alpha**2/(1+4*a**2*alpha**2) )
		F_rstep = sp.lambdify((a,drho,alpha),sp_rstep,'numpy')

		alphas = np.arange(float(xmin),float(xmax),float(xstep))
		alphas[alphas == 0] = 0.1

		self.grid = pd.DataFrame({'alpha':[],'r':[],'s':[],'t':[]})
		for alph in alphas:
		    rst = F_rstep(acf,5,alph)
		    Lrho = np.array([0-sp.sign(alph)*rst*i for i in range(rmax) if rst*i <= abs(alph)]+[
		        sp.sign(alph)*rst*i for i in range(1,rmax) if rst*i <= rnum])
		    Lalpha = np.array([alph]*Lrho.shape[0])

		    Ls = F_s(Lalpha,Lrho)
		    Lt = F_t(acf,Lalpha,Lrho)

		    self.grid = self.grid.append(pd.DataFrame({'alpha':Lalpha,'r':Lrho,'s':Ls,'t':Lt}),ignore_index=True)

		self.tree = KDTree(self.grid[['s','t']])
		# return(self.grid,self.tree)

	def transform_cylindrical(self,df,xstep,rmax,rnum,mm):

		tic = time.time()
		from sympy.abc import alpha,rho
		a,ds,dt,x = sp.symbols('a ds dt x')
		sp_dalpha = alpha*(ds + 2*a*dt*alpha)/(alpha+4*a**2*alpha**3+rho)
		F_dalpha = sp.lambdify((a,alpha,rho,ds,dt),sp_dalpha,'numpy')

		self.setup_tree(df.x.min(),df.x.max(),xstep,rmax,rnum,mm.cf[0])
		print(time.time()-tic,'Tree setup complete')

		tic = time.time()
		# Find alpha position on parabola
		nbd,nbi = self.tree.query(df[['x','z']])
		tpt = df.loc[df.index]
		npt = self.grid.loc[nbi]
		ds = tpt.x - np.array(npt.s)
		dt = tpt.z - np.array(npt.t)
		alpha = F_dalpha(mm.cf[0],np.array(npt.alpha),np.array(npt.r),ds,dt) + np.array(npt.alpha)
		alpha = alpha.astype('float64')
		print(tic-time.time(),'Alpha query complete')

		tic = time.time()
		# Calculate arclength
		x1,x2 = sp.symbols('x1 x2')
		sp_integral = (1/(2*a))*( a*x*sp.sqrt((1 + 4*a**2*x**2)) + (1/2)*sp.log(2*a*x + sp.sqrt((1 + 4*a**2*x**2))) )
		sp_arclength = sp_integral.subs(x,x2) - sp_integral.subs(x,x1)
		F_arclength = sp.lambdify((a,x1,x2),sp_arclength,'numpy')
		df['ac'] = F_arclength(np.array([mm.cf[0]]*alpha.shape[0]),np.zeros(alpha.shape[0]),alpha)
		print(tic-time.time(),'Arclength complete')

		tic = time.time()
		# Calculate theta
		df['theta'] = np.arctan2(df.y-0,df.z-mm.cf[0]*np.power(alpha,2))
		print(tic-time.time(),'theta')

		tic = time.time()
		#Calculate r
		df['r'] = np.sqrt( (df.z-mm.cf[0]*np.power(alpha,2))**2 + (df.y-0)**2 + (df.x-alpha)**2 )
		print(tic-time.time(),'r')

		return(df)

class math_model:
	'''
	Object to contain attributes associated with the math model of a sample

	:param array model: Array of coefficients calculated by np.polyfit

	.. py:attribute:: math_model.cf

		Array of coefficients for the math model

	.. py:attribute:: math_model.p

		Poly1d function for the math model to allow calculation and plotting of the model
	'''

	def __init__(self,model):

		self.cf = model
		self.p = np.poly1d(model)



#!/usr/bin/env python

import tkinter as tk
from tkinter import filedialog, messagebox
from tkinter import ttk

import os
import yaml
import copy
import cranium
import numpy as np
import time
import pandas as pd

import matplotlib
matplotlib.use('TkAgg')
from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg, NavigationToolbar2TkAgg
import matplotlib.pyplot as plt
from matplotlib.figure import Figure

class channelMaster:
	def __init__(self,parent,text,n):
		self.name = tk.Entry(parent)
		self.name.grid(row=n,column=1)
		self.name.insert(0,text)
		self.dir = None
		self.dButton = ttk.Button(parent,text='Select Folder',command=self.open_dir)
		self.dButton.grid(row=n,column=2)
		self.files = None

		self.key = n

	def open_dir(self):
		self.dir = filedialog.askdirectory()
		self.dButton['text'] = self.dir

	def return_data_dict(self):
		out = {}
		#out['files'] = self.files
		out['dir'] = self.dir
		out['name'] = self.name
		print(out)

		return(out)

class Sample:
	def __init__(self,key):
		self.key = key
		self.fs = ['','','','']
		self.outpaths = ['','','','']

		self.cbVar = tk.IntVar()
		self.degVar = tk.StringVar()
		self.directVar = tk.StringVar()

	def tab2row(self,tab,row):

		self.checkB = tk.Checkbutton(tab,text=self.key,variable=self.cbVar)
		self.checkB.select()
		self.checkB.grid(row=row,column=0)

		for i,f in enumerate(self.fs):
			ttk.Label(tab,text=f).grid(row=row,column=i+1)

	def tab3row(self,tab,r,app):

		#Sample number label
		tk.Label(tab,text=self.key).grid(row=r,column=0)

		self.plotB = tk.Button(tab,text='PCA 2D',command=lambda:self.first_alignment(app.p,app.Lc,2))
		self.plotB.grid(row=r,column=1)

		self.plotB3 = tk.Button(tab,text='PCA 3D',command=lambda:self.first_alignment(app.p,app.Lc,3))
		self.plotB3.grid(row=r,column=2)

		rotDegrees = ('-180','-90','0','90','180')
		rotDirection = ('X','Y','Z')

		self.degVar.set(rotDegrees[2])
		self.degM = tk.OptionMenu(tab,self.degVar,*rotDegrees)
		self.degM.grid(row=r,column=3)

		self.directVar.set(rotDirection[0])
		self.directM = tk.OptionMenu(tab,self.directVar,*rotDirection)
		self.directM.grid(row=r,column=4)

		self.rotB = tk.Button(tab,text='Rotate',command=self.rotate_data)
		self.rotB.grid(row=r,column=5)

		self.doneVar = tk.IntVar()
		self.doneCb = tk.Checkbutton(tab,variable=self.doneVar)
		self.doneCb.grid(row=r,column=6)

	def first_alignment(self,p,Lc,D):
		tic = time.time()

		for i,c in enumerate(Lc):
			print(i,c.name,c.dir)

		pc = self.check_settings(p)

		e = cranium.embryo(pc['expname'],self.key,pc['outdir'])
		for i,c in enumerate(Lc):
			print(i,c.dir,c.key)
			if c.dir != None:
				print(i,'data processing')
				e.add_channel(os.path.join(c.dir,self.fs[i]),c.key)
				e.chnls[c.key].preprocess_data(pc['genthresh'],[1,1,1],pc['microns'])

				#Processing for the structural channel
				if i==0:
					if D == 3:
						print('comporder',pc['comporder'])
						e.chnls[c.key].calculate_pca_median(e.chnls[c.key].raw_data,pc['medthresh'],
							pc['radius'],pc['microns'])
						pca = e.chnls[c.key].pcamed
						e.chnls[c.key].pca_transform_3d(e.chnls[c.key].df_thresh,pca,
							pc['comporder'],pc['fitdim'],deg=pc['deg'])
					elif D == 2:
						e.chnls[c.key].calculate_pca_median_2d(e.chnls[c.key].raw_data,pc['medthresh'],
							pc['radius'],pc['microns'])
						pca = e.chnls[c.key].pcamed
						e.chnls[c.key].pca_transform_2d(e.chnls[c.key].df_thresh,pca,
							pc['comporder'],pc['fitdim'],deg=pc['deg'])

					mm = e.chnls[c.key].mm
					vertex = e.chnls[c.key].vertex

				#Secondary channel processing
				if D == 3:
					e.chnls[c.key].pca_transform_3d(e.chnls[c.key].df_thresh,pca,
						pc['comporder'],pc['fitdim'],
						deg=pc['deg'],mm=mm,vertex=vertex)
				if D == 2:
					e.chnls[c.key].pca_transform_2d(e.chnls[c.key].df_thresh,pca,
						pc['comporder'],pc['fitdim'],
						deg=pc['deg'],mm=mm,vertex=vertex)

			print('sample alignment complete',time.time()-tic)

		#Save data
		for i,c in enumerate(Lc):
			if c.dir != None:
				print(i,'data saving')
				cname = c.name.get()
				for p in ':,./\\[]{};() ':
					cname = cname.replace(p,'')

				fpath = os.path.join(pc['outdir'],'_'.join([cname,pc['expname'],self.key])+'.psi')
				cranium.write_data(fpath,e.chnls[c.key].df_align)
				self.outpaths[i] = fpath

		self.plot_projections([e.chnls[Lc[0].key].df_thresh, e.chnls[Lc[0].key].df_align],mm)

	def rotate_data(self):
		deg = np.deg2rad(int(self.degVar.get()))
		print(deg)
		dM = {
			'X':np.array([[1,0,0],
					[0,np.cos(deg),-np.sin(deg)],
					[0,np.sin(deg),np.cos(deg)]]),
			'Y':np.array([[np.cos(deg),0,np.sin(deg)],
					[0,1,0],
					[-np.sin(deg),0,np.cos(deg)]]),
			'Z':np.array([[np.cos(deg),-np.sin(deg),0],
					[np.sin(deg),np.cos(deg),0],
					[0,0,1]])
		}
		M = dM[self.directVar.get()]
		print(M)

		for fp in self.outpaths:
			if fp != '':
				try:
					df = cranium.read_psi(fp)
				except:
					messagebox.showerror('Error',fp+' does not exist for rotation.')
					return

				rot = np.dot(np.array(df[['x','y','z']]),M)
				dfr = pd.DataFrame({'x':rot[:,0],'y':rot[:,1],'z':rot[:,2]})

				cranium.write_data(fp,dfr)
				print('rotating',fp)

		self.plot_projections([cranium.read_psi(self.outpaths[0])],None)
		print('plotting done')

	def plot_projections(self,Ldf,mm):
		n = len(Ldf)
		if n == 1: n=2
		fig, axarr = plt.subplots(n,3,num='Sample '+self.key) #(rows, columns)

		for i,df in enumerate(Ldf):
			df = df.sample(frac=0.01)
			axarr[i,0].scatter(df.x,df.z)
			axarr[i,1].scatter(df.x,df.y)
			axarr[i,2].scatter(df.z,df.y)

			##### To plot the model I need to account for which dimensions the model is supposed to be fit in 

			axarr[i,0].set_title('Y projection')
			axarr[i,1].set_title('Z projection')
			axarr[i,2].set_title('X projection')
			axarr[i,0].set_xlabel('X')
			axarr[i,0].set_ylabel('Z')
			axarr[i,1].set_xlabel('X')
			axarr[i,1].set_ylabel('Y')
			axarr[i,2].set_xlabel('Z')
			axarr[i,2].set_ylabel('Y')

		fig.show()

	def check_settings(self,p):

		pc = {}

		#Validate experiment name and outdir
		v = p['expname']['entry'].get()
		if v == '':
			messagebox.showerror('Error','Experiment name is not defined')
			return
		else:
			pc['expname'] = v

		if p['outdir'] == None:
			messagebox.showerror('Error','Output directory has not been selected')
			return
		else:
			pc['outdir'] = p['outdir']

		#Check median threshold value
		s = p['medthresh']['entry'].get()
		try:
			v = float(s)
			if v <= 1 and v >= 0:
				pc['medthresh'] = v
			else:
				messagebox.showerror('Error','Median threshold input must be a number between 0 and 1')
		except ValueError:
			messagebox.showerror('Error','Median threshold input must be a number between 0 and 1')
			return

		#Check radius must be integer
		s = p['radius']['entry'].get()
		try:
			pc['radius'] = int(s)
		except ValueError:
			messagebox.showerror('Error','Radius input must be an integer')
			return

		#Check genthresh float between 0 and 1
		s = p['genthresh']['entry'].get()
		try:
			v = float(s)
			if v <= 1 and v >= 0:
				pc['genthresh'] = v
			else:
				messagebox.showerror('Error','General threshold input must be a number between 0 and 1')
				return
		except ValueError:
			messagebox.showerror('Error','General threshold input must be a number between 0 and 1')
			return

		#Check microns
		lv = []
		for key in ['entry_x','entry_y','entry_z']:
			s = p['microns'][key].get()
			try:
				v = float(s)
				lv.append(v)
			except ValueError:
				messagebox.showerror('Error','Micron inputs must be numeric values')
				return
				
			pc['microns'] = lv

		#Check degree
		s = p['deg']['entry'].get()
		try:
			pc['deg'] = int(s)
		except ValueError:
			messagebox.showerror('Error','Degree input must be an integer')
			return

		#Check component order
		lv = []
		for key in ['entry_x','entry_y','entry_z']:
			s = p['comporder'][key].get()
			try:
				v = int(s)
				lv.append(v)
			except ValueError:
				messagebox.showerror('Error','Component order inputs must be numeric values')
				return
			pc['comporder'] = lv

		#Get fit dimensions
		s = p['fitdim']['entry'].get()
		if s == 'XZ Plane':
			pc['fitdim'] = ['x','z']
		elif s == 'XY Plane':
			pc['fitdim'] = ['x','y']
		elif s == 'YZ Plane':
			pc['fitdim'] = ['y','z']

		for key in pc.keys():
			print(key,pc[key])

		return(pc)



import os
import numpy as np
import cranium
import multiprocessing as mp

genthresh = 0.5
medthresh = 0.25
radius = 20
microns = [0.16,0.16,0.21]
deg = 2
comporder = [0,2,1]
fitdim = ['x','z']
scale = [1,1,1]

columns = ['x','y','z','ac','r','theta']

outdir = "D:\\Wildtype_for_modeling\\PSI"

def process(filepath):
	s = cranium.brain()
	print(filepath)
	s.read_data(filepath)

	s.preprocess_data(genthresh,[1,1,1],microns)
	s.calculate_pca_median(s.raw_data,medthresh,radius,microns)

	s.pca_transform_3d(s.df_thresh,s.pcamed,comporder,fitdim)
	s.transform_coordinates()

	fname = filepath.split('\\')[-1].split('_')
	name = fname[0]+'_'+fname[1]+'_'+fname[2]+'.psi'

	cranium.write_data(os.path.join(outdir,name),s.df_align[columns])

	print(name)

ddir = "D:\\Wildtype_for_modeling\\Prob"

if __name__=='__main__':

	num=[]
	for f in os.listdir(ddir):
		if 'Probabilities' in f and 'h5' in f:
			num.append(os.path.join(ddir,f))

	for i in range(0,len(num),5):
		if i+5>len(num):
			L = num[i:len(num)]
		else:
			L = num[i:i+5]

		pool = mp.Pool()
		pool.map(process,L)
		pool.close()
		pool.join()


import deltascope as cranium
import multiprocessing as mp
import time
import os
import sys
from functools import partial
from random import randint
import numpy as np
import pandas as pd
import re

def transform_file(f,model=None,test=False,tri=False):

	print(f, 'starting')

	tic = time.time()

	s = cranium.brain()
	df = cranium.read_psi(f)
	if 'ac' not in df.columns:

		# if tri==True:
		# 	s.df_align = df[['x','y','z']]
		# 	pt1x = np.max(s.df_align.x)
		# 	pt1z = s.df_align[s.df_align.x == pt1x].z.values[0]
		# 	pt2x = np.min(s.df_align.x)
		# 	pt2z = s.df_align[s.df_align.x == pt2x].z.values[0]
		#
		# 	s.mm = cranium.math_model(np.polyfit([pt1x,0,pt2x],[pt1z,0,pt2z],2))
		#
		# 	if test==True:
		# 		return(s,[pt1x,pt2x],[pt1z,pt2z])

		# if tri==False:
		s.df_align = df[['x','y','z']]
		snum  = re.findall(r'\d+',f.split('.')[0])[0]
		s.mm = cranium.math_model(model.loc[int(snum)].values)
		num = int(snum)
			# return(s)

		# if test==False:
		s.transform_coordinates()

		cranium.write_data(f,s.df_align)

	print(f,'complete',time.time()-tic)


if __name__=='__main__':

	outdirs = ['D:\\HSs1a_yot_expt\\zrf1\\hss1a\\Output-08-13']

	for outdir in outdirs:

		files = os.listdir(outdir)
		files.remove('model.csv')

		os.chdir(outdir)

		model = pd.read_csv(os.path.join(outdir,'model.csv'),index_col='Unnamed: 0')
		transform = partial(transform_file,model=model)

		n = len(files)
		for i in range(0,n,5):
			if i+5>n:
				L = files[i:n]
			else:
				L = files[i:i+5]

			pool = mp.Pool()
			pool.map(transform,L)
			pool.close()
			pool.join()

		print('Processing complete')



#!/usr/bin/env python

import tkinter as tk
from tkinter import filedialog, messagebox
from tkinter import ttk

import os
import yaml
import copy
import cranium
import numpy as np
import time
from appClass import *

import matplotlib
matplotlib.use('TkAgg')
from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg, NavigationToolbar2TkAgg
import matplotlib.pyplot as plt
from matplotlib.figure import Figure

class Application(tk.Frame):
	def __init__(self,master=None):
		tk.Frame.__init__(self,master)

		self.grid(sticky=tk.N+tk.S+tk.E+tk.W)
		self.createNotebook()
		self.s = ttk.Style()

		#Create none variables so that app can always be saved
		self.fnums, self.Lnums = None,None

	def createNotebook(self):
		'''Create notebook to contain different tabs with data processing steps'''
		self.note = ttk.Notebook(self)
		self.note.grid()

		self.tab1 = ttk.Frame(self.note)
		self.note.add(self.tab1,text='Channels')
		self.tab2 = ttk.Frame(self.note)
		self.note.add(self.tab2,text='Files',state='disabled')
		self.tab25 = ttk.Frame(self.note)
		self.note.add(self.tab25,text='Settings',state='normal')
		self.tab3 = ttk.Frame(self.note)
		self.note.add(self.tab3,text='Alignment',state='disabled')

		self.createTab1()
		self.createTab2()
		self.createTab25()
		self.createTab3()

	def createTab1(self):
		'''Create tab that allows user to define the source directory for data'''

		#Deactivated next button that turns on after user adds data to channel 1
		self.nextB = tk.Button(self.tab1,text='Next',command=lambda:self.note.select(self.tab2),state=tk.DISABLED)
		self.nextB.grid(row=5,column=2)

		#Initialize channel objects to support selection of source directories
		self.cs = channelMaster(self.tab1,'Structural channel: ',0)
		self.cs.dButton.bind('<Button-1>',lambda e:self.activate(self.nextB,self.note,self.tab2))
		self.c2 = channelMaster(self.tab1,'Channel 2: ',1)
		self.c3 = channelMaster(self.tab1,'Channel 3: ',2)
		self.c4 = channelMaster(self.tab1,'Channel 4: ',3)
		self.c5 = channelMaster(self.tab1,'Channel 5: ',4)
		self.Lc = [self.cs,self.c2,self.c3,self.c4,self.c5]

		#Loads saved files
		self.loadB = tk.Button(self.tab1,text='Load saved file',command=self.load_status)
		self.loadB.grid()

	def createTab2(self):
		'''Tab 2 displays a list of files found in source directory
		and has check boxes for selection'''
		
		#Generate file list button
		self.filegenB = tk.Button(self.tab2,text='Generate file list',command=self.filegen)
		self.filegenB.grid(row=0)

		#Save button
		self.saveB = tk.Button(self.tab2,text='Save',command=self.save_status)
		self.saveB.grid(row=0,column=1)

		#Go to next page button
		self.nextB2 = tk.Button(self.tab2,text='Next',command=lambda:self.next_tab(self.tab25))
		self.nextB2.grid(row=0,column=2)

	def createTab25(self):
		'''Text entry objects for all parameters with examples already entered'''

		##### Tab 2.5 #####
		r = 0
		self.outdir = None
		self.outdirE = tk.Button(self.tab25,text='Final Output Folder: ',command=self.select_outdir)
		self.outdirE.grid(row=r,column=0)

		self.nextB25 = tk.Button(self.tab25,text='Next',command=lambda:self.next_tab(self.tab3))
		self.nextB25.grid(row=0,column=2)

		#Info in dict: row,title,example,help text
		self.p = {
			'expname': {
				'row': 1,
				'title': 'Experiment name: ',
				'example': 'experiment'
			},
			'medthresh': {
				'row': 2,
				'title': 'Median filter threshold: ',
				'example': '0.25'
			},
			'radius': {
				'row': 3,
				'title': 'Median filter radius: ',
				'example': '20',
			},
			'genthresh': {
				'row': 4,
				'title': 'General threshold: ',
				'example': '0.5'
			},
			'microns': {
				'row': 5,
				'title': 'Micron dimensions of voxel: ',
				'example': ['0.16','0.16','0.21']
			},
			'deg': {
				'row': 6,
				'title': 'Degree of fitted function: ',
				'example': '2'
			},
			'comporder': {
				'row': 7,
				'title': 'Assignment order of principle components: ',
				'example': ['0','2','1']
			},
			'fitdim': {
				'row': 8,
				'title': 'Dimensions to fit function: ',
				'example': 'x,z'
			}
		}

		#Create text entry objects
		for key in self.p.keys():
			#Triple entry
			if key in ['microns','comporder']:
				tk.Label(self.tab25,text=self.p[key]['title']).grid(row=self.p[key]['row'],column=0)

				tk.Label(self.tab25,text='X:').grid(row=self.p[key]['row'],column=1)
				self.p[key]['entry_x'] = tk.Entry(self.tab25)
				self.p[key]['entry_x'].grid(row=self.p[key]['row'],column=2)
				self.p[key]['entry_x'].insert(0,self.p[key]['example'][0])

				tk.Label(self.tab25,text='Y:').grid(row=self.p[key]['row'],column=3)
				self.p[key]['entry_y'] = tk.Entry(self.tab25)
				self.p[key]['entry_y'].grid(row=self.p[key]['row'],column=4)
				self.p[key]['entry_y'].insert(0,self.p[key]['example'][1])

				tk.Label(self.tab25,text='Z:').grid(row=self.p[key]['row'],column=5)
				self.p[key]['entry_z'] = tk.Entry(self.tab25)
				self.p[key]['entry_z'].grid(row=self.p[key]['row'],column=6)
				self.p[key]['entry_z'].insert(0,self.p[key]['example'][2])

			#Drop down fit dim
			elif key == 'fitdim':
				tk.Label(self.tab25,text=self.p[key]['title']).grid(row=self.p[key]['row'],column=0)

				options = ('XZ Plane','XY Plane', 'YZ Plane')
				self.p[key]['entry'] = tk.StringVar()
				
				self.p[key]['entry'].set(options[0])

				self.p[key]['menu'] = tk.OptionMenu(self.tab25,self.p[key]['entry'],*options)
				self.p[key]['menu'].grid(row=self.p[key]['row'],column=1)

			#Single entry
			else:
				tk.Label(self.tab25,text=self.p[key]['title']).grid(row=self.p[key]['row'],column=0)
				self.p[key]['entry'] = tk.Entry(self.tab25)
				self.p[key]['entry'].grid(row=self.p[key]['row'],column=1)
				self.p[key]['entry'].insert(0,self.p[key]['example'])

		self.p['outdir'] = None

	def createTab3(self):

		##### Tab 3 #####
		# self.plotB = tk.Button(self.tab3,text='plot',command=lambda:self.plot_projection('06'))
		# self.plotB.grid(row=1)

		self.pBar = ttk.Progressbar(self.tab3)
		self.pBar.grid(row=0,column=1)

		tk.Label(self.tab3,text='Rotate by ___ degrees:').grid(row=0,column=3)
		tk.Label(self.tab3,text='around ___ axis:').grid(row=0,column=4)
		tk.Label(self.tab3,text='Done?').grid(row=0,column=6)

		
	def activate(self,button,nb,ntab):
		button['state'] = tk.NORMAL
		button['bg'] = 'green'
		nb.tab(ntab,state='normal')

	def next_tab(self,ntab):
		self.note.tab(ntab,state='normal')
		self.note.select(ntab)

	def select_outdir(self):
		self.p['outdir'] = filedialog.askdirectory()
		tk.Label(self.tab25,text=self.p['outdir']).grid(row=0,column=1)

	def filegen(self):
		self.fnums = {}
		self.Lnums = []
		
		for i,c in enumerate(self.Lc):
			if c.dir != None and c.dir != '':
				files = os.listdir(c.dir)
			else:
				files = []

			c.files = []
			for f in files:
				if 'Probabilities' in f and 'h5' in f:
					key = f.split('_')[-2]
					if i == 0:
						self.fnums[key] = Sample(key)
						self.fnums[key].fs[i] = f
						self.Lnums.append(int(key))
					else:
						self.fnums[key].fs[i] = f

		self.Lnums.sort()
		self.display_files()

	def display_files(self):

		#Header 
		ttk.Label(self.tab2,text='Use?').grid(row=1,column=0)
		#ttk.Label(self.tab2,text='Number').grid(row=1,column=2)
		ttk.Label(self.tab2,text='Structural').grid(row=1,column=1)
		ttk.Label(self.tab2,text='Channel 2').grid(row=1,column=2)
		ttk.Label(self.tab2,text='Channel 3').grid(row=1,column=3)
		ttk.Label(self.tab2,text='Channel 4').grid(row=1,column=4)

		for i,n in enumerate(self.Lnums):
			key = str(n)
			if len(key) == 1:
				key = '0'+ key
			self.fnums[key].tab2row(self.tab2,i+2)
			self.fnums[key].tab3row(self.tab3,i+2,self)


	def out_fnums(self):
		for key in self.fnums.keys():
			print(self.fnums[key][-1].get())
		print('******')

	def save_status(self):

		fpath = filedialog.asksaveasfilename(defaultextension=".yml")
		print(fpath)

		out = {}
		out['curtab'] = self.note.tab(self.note.select(),'text')

		for i,c in enumerate([self.cs,self.c2,self.c3,self.c4]):
			if c == None:
				out['c'+str(i)] = None
			else:
				out['c'+str(i)] = c.return_data_dict()

		out['Lnums'] = self.Lnums

		out['fnums'] = {}
		for key in self.fnums.keys():
			line = copy.deepcopy(self.fnums[key][:-1])
			line.append(self.fnums[key][-1].get())
			out['fnums'][key] = line

		print(out)
		print(self.fnums)

		f = open(fpath,'w')
		yaml.dump(out,f)
		f.close()

	def load_status(self):

		fpath = filedialog.askopenfilename()
		print(fpath)
		s = open(fpath,'r').read()
		d = yaml.load(s)

		varbind = {
			'curtab': {'Channels':self.tab1,'Files':self.tab2,'Alignment':self.tab3},
			'fnums': self.fnums,
			'Lnums': self.Lnums,
			'c0': self.cs,
			'c1': self.c2,
			'c2': self.c3,
			'c3': self.c4
		}

		n = 0
		for key in d.keys():
			if key == 'Lnums':
				self.Lnums = d[key]
			elif key == 'fnums':
				if d['fnums'] != None:
					self.fnums = {}
					for key in d['fnums'].keys():
						self.fnums[key] = d['fnums'][key][:-1]
						self.fnums[key].append(tk.IntVar().set(d['fnums'][key][-1]))
			elif key != 'curtab':
				if d[key] == None:
					varbind[key] = d[key]
				else:
					varbind[key].dir = d[key]['dir']
					varbind[key].dButton['text'] = d[key]['dir']
					varbind[key].name = d[key]['name']


		if d['curtab'] in ['Files','Alignment']:
			self.activate(self.nextB,self.note,self.tab2)
		if self.fnums != None:
			self.display_files()
		self.note.select(varbind['curtab'][d['curtab']])
		print('load complete')

if __name__=='__main__':
	app = Application()
	app.master.title('Sample Application')
	app.mainloop()


import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
import os
import re
import h5py
import deltascope as cranium

def read_h5prob_to_dict(fdir):
    '''
    Read in hdf5 array from probability file into dictionary

    :param str fdir: Filepath to directory
    :return: Dictionary with filenumber as key and raw data as item
    '''
    D = {}
    for f in os.listdir(fdir):
        if 'h5' in f:
            num = re.findall(r'\d+',f.split('.')[0])[1]
            file = h5py.File(os.path.join(fdir,f))
            D[num] = file.get('exported_data')

    return(D)

def preprocess(fpath,p,stop=None,pca=None,mm=None,vertex=None):
    '''
    Generate brain object and process until stop is triggered

    :param str stop: 'df_thresh' 'median' 'df_align'
    :return: Cranium brain object
    '''

    b = cranium.brain()
    b.read_data(fpath)
    b.preprocess_data(p['gthresh'],p['scale'],p['microns'])
    if stop == 'df_thresh':
        return(b)

    if pca == None:
        b.calculate_pca_median(b.raw_data,p['mthresh'],p['radius'],p['microns'])
        pca = b.pcamed
    if stop == 'median':
        return(b)

    b.pca_transform_3d(b.df_thresh,pca,p['comp_order'],p['fit_dim'],deg=p['deg'],mm=mm,vertex=vertex)
    if (stop == 'df_align') | (stop == None):
        return(b)

def plot_lines(ax,i):
    '''
    Plots origin lines on graph

    :param plt.subplots ax: Subplot array with minimum dimension [2,3]
    :param int i: Index of row that should be labeled
    '''

    ax[i,0].axhline(0,c='y')
    ax[i,0].axvline(0,c='r')
    ax[i,1].axhline(0,c='y')
    ax[i,1].axvline(0,c='c')
    ax[i,2].axhline(0,c='c')
    ax[i,2].axvline(0,c='r')

def scatter_df(ax,i,df):
    '''
    Scatter plot of df with projections in all 3 axes

    :param plt.subplots ax: Subplot array with minimum dimension [2,3]
    :param int i: Index of row that should be labeled
    :param pd.DataFrame df: Dataframe containing 'x','y','z' columns preferably downsampled for faster plotting
    '''

    ax[i,0].scatter(df.x,df.y,s=10)
    ax[i,1].scatter(df.x,df.z,s=10)
    ax[i,2].scatter(df.z,df.y,s=10)

def imshow_arr(ax,i,arr,f):
    '''
    Scatter plot of df with projections in all 3 axes according to supplied function f

    :param plt.subplots ax: Subplot array with minimum dimension [2,3]
    :param int i: Index of row that should be labeled
    :param np.array arr: 3D image array
    :param np.fxn f: Numpy function that takes an array and an axis value as parameters, usually np.min or np.max
    '''

    ax[i,0].imshow(f(arr,axis=0),cmap='Greys')
    ax[i,1].imshow(f(arr,axis=1),cmap='Greys')
    ax[i,2].imshow(f(arr,axis=2),cmap='Greys')

def rotate(df,A):
    '''
    Transform dataframe according to matrix A

    :param pd.DataFrame df: Dataframe containing 'x','y','z'
    :param np.array A: 3x3 array containing transformation matrix
    :return: New dataframe with transformed data
    '''

    rt = np.dot(np.array(df[['x','y','z']]),A)
    dfo = pd.DataFrame({'x':rt[:,0],'y':rt[:,1],'z':rt[:,2]})
    return(dfo)

def save_at(k,df,outdir,expname):
    '''
    Save dataframe as psi file in outdir

    :param str k: Sample number
    :param pd.DataFrame df: Dataframe containing 'x','y','z'
    :param str outdir: Filepath to destination directory for psi
    :param str expname: Descriptive name for experiment to be used in filename
    '''
    cranium.write_data(os.path.join(outdir,'AT_'+k+'_'+expname+'.psi'),df)

def save_zrf(k,df,outdir,expname):
    '''
    Save dataframe as psi file in outdir

    :param str k: Sample number
    :param pd.DataFrame df: Dataframe containing 'x','y','z'
    :param str outdir: Filepath to destination directory for psi
    :param str expname: Descriptive name for experiment to be used in filename
    '''
    cranium.write_data(os.path.join(outdir,'ZRF_'+k+'_'+expname+'.psi'),df)

def save_both(k,dfa,dfz,outdir,expname):
    '''
    Save dataframe as psi file in outdir

    :param str k: Sample number
    :param pd.DataFrame dfa: AT dataframe containing 'x','y','z'
    :param pd.DataFrame dfa: Zrf dataframe containing 'x','y','z'
    :param str outdir: Filepath to destination directory for psi
    :param str expname: Descriptive name for experiment to be used in filename
    '''
    save_at(k,dfa,outdir,expname)
    save_zrf(k,dfz,outdir,expname)

def save_caax(k,df,outdir,expname):

    cranium.write_data(os.path.join(outdir,'caax_'+k+'_'+expname+'.psi'),df)

def save_all(k,dfa,dfz,dfc,outdir,expname):
    '''
    Save dataframe as psi file in outdir

    :param str k: Sample number
    :param pd.DataFrame dfa: AT dataframe containing 'x','y','z'
    :param pd.DataFrame dfa: Zrf dataframe containing 'x','y','z'
    :param str outdir: Filepath to destination directory for psi
    :param str expname: Descriptive name for experiment to be used in filename
    '''
    save_at(k,dfa,outdir,expname)
    save_zrf(k,dfz,outdir,expname)
    save_caax(k,dfc,outdir,expname)

######### Alignment functions ###############

def calc_rotation(pts,d):
    '''
    Calculate a rotation matrix A based on two anchor points

    :param pd.DataFrame pts: Two points in x and the dimension d
    :param str d: Second dimension of the two points: 'z' or 'y'
    :return: midpoint, rotation matrix
    '''
    dx = (pts.iloc[0].x - pts.iloc[1].x)/2
    dy = (pts.iloc[0][d] - pts.iloc[1][d])/2

    mp = [pts.iloc[1].x + dx, pts.iloc[1][d] + dy]

    phi = np.arctan2(dy,dx)
    if phi > np.pi/2:
        phi = -(np.pi - phi)
    if d == 'y':
        A = np.array([[np.cos(phi),-np.sin(phi),0],
        			[np.sin(phi),np.cos(phi),0],
        			[0,0,1]])
    else:
        A = np.array([[np.cos(phi),0,-np.sin(phi)],
                    [0,1,0],
                    [np.sin(phi),0,np.cos(phi)]])

    return(mp,A)

def xrotate(df,Ldf,d,pts=None):
    '''
    Define two anchor points and rotate data so that the anchor points are level

    :param pd.DataFrame df: Dataframe that will be used to calculate the anchor points and rotation
    :param list Ldf: List of additionally dataframes that should be rotated based on the calculation from `df`
    :param str d: Second dimension of the plane that needs to be rotated: 'z' or 'y'
    :param pd.DataFrame pts: Two points in x and the dimension d
    :return: `df` after rotation, a list of rotated dataframes from `Ldf`, `pts`
    '''

    if type(pts) == type(None):
        pt1,pt2 = cranium.find_anchors(df,d)
        pts = pd.DataFrame({'x':[pt1['x'],pt2['x']],d:[pt1[d],pt2[d]]})

    mp,A = calc_rotation(pts,d)

    out = rotate(df,A)
    Lout = []
    for d in Ldf:
        Lout.append(rotate(d,A))

    return(out,Lout,pts)

def zyswitch(df,Ldf):
    '''
    Switch z and y axes by reassigning z to y and vice versa

    :param pd.DataFrame df: Dataframe that will be used to calculate the anchor points and rotation
    :param list Ldf: List of additionally dataframes that should be rotated based on the calculation from `df`
    :return: `df` and `Ldf` after exchanging z and y
    '''

    out = pd.DataFrame({'x':df.x,'y':df.z,'z':df.y})
    Lout = []
    for d in Ldf:
        Lout.append(pd.DataFrame({'x':d.x,'y':d.z,'z':d.y}))

    return(out,Lout)

def vertex(df,Ldf,pts=None):
    '''
    Calculate the vertex in the XZ plane and shift the vertex to the origin

    :param pd.DataFrame df: Dataframe that will be used to calculate the anchor points and rotation
    :param list Ldf: List of additionally dataframes that should be rotated based on the calculation from `df`
    :return: `df` and `Ldf` after translation of the vertex to the origin
    '''

    if type(pts)!=type(None):
        cf = np.polyfit(pts.x,pts.z,2)
    else:
        cf = np.polyfit(df.x,df.z,2)
    x = -cf[1]/(2*cf[0])
    z = np.poly1d(cf)(x)
    y = np.mean(df.y)

    out = pd.DataFrame({
        'x':df.x-x,
        'y':df.y-y,
        'z':df.z-z
    })
    Lout = []
    for d in Ldf:
        Lout.append(pd.DataFrame({
            'x':d.x-x,
            'y':d.y-y,
            'z':d.z-z
        }))

    if type(pts)!=type(None):
        cfout = np.polyfit(pts.x-x,pts.z-z,2)
    else:
        cfout = np.polyfit(out.x,out.z,2)

    return(out,Lout,cfout)

def flip(df,Ldf):
    '''
    Rotate by 180 degrees around the x axis

    :param pd.DataFrame df: Dataframe that will be used to calculate the anchor points and rotation
    :param list Ldf: List of additionally dataframes that should be rotated based on the calculation from `df`
    :return: `df` and `Ldf` after rotation
    '''

    A = np.array([[1,0,0],
                [0,np.cos(np.pi),-np.sin(np.pi)],
                [0,np.sin(np.pi),np.cos(np.pi)]])

    out = rotate(df,A)
    Lout = []
    for d in Ldf:
        Lout.append(rotate(d,A))

    return(out,Lout)

def yzrotate(df,Ldf,mm=None):
    '''
    Fit a line in YZ and rotate so the line is horizontal

    :param pd.DataFrame df: Dataframe that will be used to calculate the anchor points and rotation
    :param list Ldf: List of additionally dataframes that should be rotated based on the calculation from `df`
    :return: `df` and `Ldf` after rotation to correct the lines, 1d line function, range of z values for the line
    '''

    if mm == None:
        mm = np.polyfit(df.z,df.y,1)
    p = np.poly1d(mm)
    xrange = np.arange(1.1*np.min(df.z), 1.1*np.max(df.z))
    phi = -np.arctan(mm[0])
    A = np.array([[1,0,0],
                [0,np.cos(phi),-np.sin(phi)],
                [0,np.sin(phi),np.cos(phi)]])

    out = rotate(df,A)
    Lout = []
    for d in Ldf:
        Lout.append(rotate(d,A))

    return(out,Lout,p,xrange)

def check_yz(df,Ldf,mm=None):
    '''
    Fit line in YZ and rotate line to be horizontal using `yzrotate` then display graph

    :param pd.DataFrame df: Dataframe that will be used to calculate the anchor points and rotation
    :param list Ldf: List of additionally dataframes that should be rotated based on the calculation from `df`
    :return: `df` and `Ldf` after rotation to correct the lines, array of graph subplots, 1d line function
    '''

    out,Lout,p,xrange = yzrotate(df,Ldf,mm=mm)
    ax = make_graph([df,out]+Lout)
    ax[0,2].plot(xrange,p(xrange),c='m')

    return(out,Lout,ax,p)

def make_graph(Ldf,Lim=[]):
    '''
    Plot the three projections of each dataframe in `Ldf`

    :param list Ldf: List of additionally dataframes that should be rotated based on the calculation from `df`
    :return: Array of subplots with minimum dimension of 2x3
    '''

    n = len(Ldf)+len(Lim)
    fig,ax = plt.subplots(n,3,subplot_kw={'aspect':'equal','adjustable':'datalim'},figsize=(12,n*4))

    for i,d in enumerate(Ldf):
        scatter_df(ax,i,d.sample(frac=0.1))
        plot_lines(ax,i)

    for i,im in enumerate(Lim):
        imshow_arr(ax,len(Ldf)+i,im,np.min)

    return(ax)

def start(k,Da,LD,im=False):
    '''
    Extract `df_align` dataframes from `cranium.brain` objects and plots using `make_graph`

    :param str k: Key corresponding to the relevant sample
    :param dict Da: Dictionary containing brain objects for each sample with key `k`
    :param list LD: List of any additional dictionaries from which `df_align` should be extracted
    :returns: key, primary dataframe, list of additional dataframes, subplot array
    '''
    Lim = []
    Lim.append(Da[k].raw_data)
    df = Da[k].df_align
    Ldf = []
    for D in LD:
        Ldf.append(D[k].df_align)
        Lim.append(D[k].raw_data)

    if im==True:
        ax = make_graph([df]+Ldf,Lim)
    else:
        ax = make_graph([df]+Ldf,[])
    return(k,df,Ldf,ax)

def check_pts(df,Ldf,d):
    '''
    Calculates anchor points and plots before and after rotation

    :param pd.DataFrame df: Dataframe that will be used to calculate the anchor points and rotation
    :param list Ldf: List of additionally dataframes that should be rotated based on the calculation from `df`
    :param str d: Second dimension of the plane that needs to be rotated: 'z' or 'y'
    :returns: Rotated `df`, list of rotated dfs from `Ldf`, anchor points, subplot array
    '''
    df1,Ldf1,pts = xrotate(df,Ldf,d)
    ax = make_graph([df,df1]+Ldf1)
    if d=='z':
        ax[0,1].scatter(pts.x,pts[d],c='r')
    if d=='y':
        ax[0,0].scatter(pts.x,pts[d],c='r')
    return(df1,Ldf1,pts,ax)

def revise_pts(df,Ldf,d,pts):
    '''
    Rotate and plot data based on `pts` parameter

    :param pd.DataFrame df: Dataframe that will be used to calculate the anchor points and rotation
    :param list Ldf: List of additionally dataframes that should be rotated based on the calculation from `df`
    :param str d: Second dimension of the plane that needs to be rotated: 'z' or 'y'
    :param pd.DataFrame pts: Two points in x and the dimension d
    :returns: Rotated `df`, list of rotated dfs from `Ldf`, subplot array
    '''
    df1,Ldf1,pts2 = xrotate(df,Ldf,d,pts=pts)
    ax = make_graph([df,df1]+Ldf1)
    if d=='z':
        ax[0,1].scatter(pts2.x,pts2[d],c='r')
    if d=='y':
        ax[0,0].scatter(pts2.x,pts2[d],c='r')
    return(df1,Ldf1,ax)

def ch_vertex(df,Ldf,pts=None):
    '''
    Calculate vertex based on `df` and translate all dataframes to shift vertex to origin

    :param pd.DataFrame df: Dataframe that will be used to calculate the anchor points and rotation
    :param list Ldf: List of additionally dataframes that should be rotated based on the calculation from `df`
    :returns: Translated `df`, list of translated dataframes from `Ldf`, subplot array
    '''
    df1,Ldf1,mm = vertex(df,Ldf,pts=pts)
    ax = make_graph([df1]+Ldf1)
    xrange = np.arange(df1.x.min(),df1.x.max())
    ax[0,1].plot(xrange,np.poly1d(mm)(xrange),c='m')
    return(df1,Ldf1,mm,ax)



from tkinter import Tk, Label, Button, filedialog
from tkinter import ttk


class Gui:

	def __init__(self,master):
		self.master = master
		master.title('simple gui')

		ttk.Style().configure("TNotebook", background='b')
		ttk.Style().map("TNotebook.tab",background=[('selected','r')])
		# Style().configure('TNotebook.Tab')

		note = ttk.Notebook(root)
		self.tab1 = page(self.master,note,'1')
		self.tab1.set_instruction('Select directories with raw data')
		Button(self.tab1.tab,text='Select Channel 1',command=self.open_dir1).grid(row=2,column=1)
		Button(self.tab1.tab,text='Select Channel 2',command=self.open_dir2).grid(row=2,column=2)

		self.tab2 = page(self.master,note,'2')
		note.grid(row=1,columnspan=2)

		Button(self.tab2.tab,text='Identify sample numbers',command=self.calc_num).grid(row=3,columnspan=2)

		# self.greet_button = Button(master,text="greet", command = self.greet)
		# self.greet_button.pack()

		# self.close_button = Button(master, text="close", command=master.quit)
		# self.close_button.pack()

		# self.open_button = Button(master, text='Open', command=self.open_dir)
		# self.open_button.pack()

		# tkvar = StringVar(root)
		# choices = {'a','b','c'}
		# tkvar.set('a')

		# popupMenu = OptionMenu(self.master, tkvar, *choices)
		# popupMenu.pack()

	def open_dir1(self):
		self.c1dir = filedialog.askdirectory()
		Label(self.tab2.tab,text='Channel 1:').grid(row=1,column=1)
		Label(self.tab2.tab,text=self.c1dir).grid(row=2,column=1)

	def open_dir2(self):
		self.c2dir = filedialog.askdirectory()
		Label(self.tab2.tab,text='Channel 2:').grid(row=1,column=2)
		Label(self.tab2.tab,text=self.c2dir).grid(row=2,column=2)

	def calc_num(self):
		files = os.listdir(self.c1dir)
		self.s_num = []

		for f in files:
			if 'h5' in f and 'Probabilities' in f:
				self.s_num.append(f.split('_')[1])
				


class page():

	def __init__(self,master,note,label):
		self.master = master
		self.note = note

		self.tab = ttk.Frame(note)
		self.note.add(self.tab,text=label)

	def set_instruction(self,text):

		self.instr = Label(self.tab,text=text)
		self.instr.grid(row=1)

	def activate(self):
		self.note.tab(self.tab,state='normal')

	def deactivate(self):
		self.note.tab(self.tab,state='disabled')

root = Tk()
my_gui = Gui(root)
root.mainloop()



#import plotly

#plotly.tools.set_credentials_file(username='msschwartz21', api_key='OrM0hMDBvseeT6SCjxNb')

def process_data(path):

	tic = time.time()

	s = cranium.brain()
	s.read_data(path)
	s.create_dataframe()
	s.pca_double_transform(0.5,['x','z'])
	s.transform_coordinates()

	columns = ['x','y','z','ac','r','theta']
	cranium.write_data(path[:-3]+'.psi',s.df_align[columns])

	toc = time.time()
	print(toc-tic)

if __name__=='__main__':

	root = 'C:\\Users\\zfishlab\\Desktop\\zrf1wt13umyot21um'
	stypes = ['wt']#,'yot'] #['wt','yot']
	chs = ['AT2\\Prob','ZRF1']
	prefixes = ['AT','ZRF1']

	#Create outdir
	outdir = os.path.join(root,'Output'+str(randint(10,99)))
	os.mkdir(outdir)
	print('outdir',outdir)

	for stype in stypes:
		d = os.path.join(root,stype,chs[0])
		print(stype,'starting')

		files = os.listdir(d)
		#Get sample numbers
		nums = []
		for file in files:
			if 'h5' in file and 'Probabilities' in file:
				nums.append(os.path.join(d,file))

		print(nums)

		processfxn = partial(cranium.process_sample,
			root=os.path.join(root,stype),
			outdir=outdir,
			name=stype,
			chs=chs,
			prefixes=prefixes,
			threshold=0.5,
			scale=[100000,1,1000],
			deg=2,
			primary_key='at',
			comp_order=[0,2,1],
			fit_dim=['x','z'],
			flip_dim='z')

		n = len(nums)
		for i in range(0,n,5):
			if i+5 > n:
				L = nums[i:n]
			else:
				L = nums[i:i+5]

			pool = mp.Pool()
			pool.map(process_data,L)
			pool.close()
			pool.join()

		print(stype, 'complete')


	# for d in dirs:
	# 	print(d)
	# 	files = os.listdir(d)
		
	# 	#Check that all files are hdf5
	# 	hfiles = []
	# 	for file in files:
	# 		if 'h5' in file and 'Probabilities' in file:
	# 			hfiles.append(os.path.join(d,file))

	# 	pool = mp.Pool()
	# 	pool.map(cranium.process_sample,hfiles)
	# 	pool.close()
	# 	pool.join()

	# 	print(d,'complete')



import cranium

expname = 'test'
samplenum = '01'
directory = 'data'

#Create an embryo object that facilitates data processing
e = cranium.embryo(expname,samplenum,directory)

#For each channel in your sample, add a channel with a unique name, e.g. 'c1' or 'c2'
e.add_channel('data/C1/AT_01_Probabilities.h5','c1')
# e.add_channel('data/C2/ZRF1_01_Probabilities.h5','c2')

genthresh = 0.5
scale = [1,1,1]
microns = [0.16,0.16,0.21]

#Threshold each channel and scale points according to voxel dimension in microns
e.chnls['c1'].preprocess_data(genthresh,scale,microns)
# e.chnls['c2'].preprocess_data(genthresh,scale,microns)

medthresh = 0.25
radius = 20
microns = [0.16,0.16,0.21]

#Run PCA on the structural channel, in this case, c1
e.chnls['c1'].calculate_pca_median(e.chnls['c1'].raw_data,medthresh,radius,microns)

#Save the pca object that includes the transformation matrix
pca = e.chnls['c1'].pcamed

comporder = [0,2,1]
fitdim = ['x','z']

#Transform the structural channel using the saved pca object
e.chnls['c1'].pca_transform_3d(e.chnls['c1'].df_thresh,pca,comporder,fitdim,deg=2)

#Save the mathematical model and vertex (center point) of the structural channel
mm = e.chnls['c1'].mm
vertex = e.chnls['c1'].vertex

#Transform any additional channels using the pca object calculated based on the structural channel
# e.chnls['c2'].pca_transform_3d(e.chnls['c2'].df_thresh,pca,comporder,fitdim,deg=2,mm=mm,vertex=vertex)

#Transform each channel to cylindrical coordinates
e.chnls['c1'].transform_coordinates()
# e.chnls['c2'].transform_coordinates()

#Save processed data to .psi file
# e.save_psi()



import setuptools


def readme():
    with open('README.rst') as f:
        return (f.read())


def requirements():
    with open('requirements.txt') as f:
        return (f.read().split('\n'))


setuptools.setup(name='deltascope',
                 version='0.2.6',
                 description='Python package to quantify biological structure',
                 long_description=readme(),
                 keywords='biology image analysis quantification',
                 url='https://github.com/msschwartz21/deltascope',
                 author='Morgan Schwartz',
                 author_email='msschwartz21@gmail.com',
                 license='GNU GPL',
                 packages=['deltascope'],
                 install_requires=requirements(),
                 include_package_data=True,
                 classifiers=[
                     "License :: OSI Approved :: GNU General Public License (GPL)",
                     "Programming Language :: Python :: 3.7"
                 ]
                 )
